from utils import Logger, LivePlotter
from nmotion_transport import Actuator, IMU, Driver, USBInterface, getCurrentLibraryVersion
from device_manager import DeviceManager
import IPython
import gc

CLI_VERSION = "0.1.1-beta"

def __print_banner():
    print(f'NMotion CLI Tool v{CLI_VERSION} ({getCurrentLibraryVersion()})')
    print('Please connect your NMotion Component via NLink device.')
    print('You can also type help() or quit().')

def __on_new_device_callback(dev_id: int, dev_type: int, interface: USBInterface) -> None:
    if(dev_type == 1):
        __logger.notify(f"New Device with ID: {dev_id} got connected. Actuator_{dev_id} object created.")
        globals()[f'Actuator_{dev_id}'] = Actuator(dev_id, interface) 
    elif(dev_type == 2):
        __logger.notify(f"New Device with ID: {dev_id} got connected. Driver_{dev_id} object created.")
        globals()[f'Driver_{dev_id}'] = Driver(dev_id, interface)     
    elif(dev_type == 3):
        __logger.notify(f"New Device with ID: {dev_id} got connected. IMU_{dev_id} object created.")
        globals()[f'IMU_{dev_id}'] = IMU(dev_id, interface)     
    else:
        __logger.warn(f'Unknown Device of type {dev_type} with ID: {dev_id} got connected. No object will be created.')

def __on_remove_device_callback(dev_id: int, dev_type: int) -> None:
    __logger.warn(f"Device with ID: {dev_id} is removed.")
    if(dev_type == 1):
        del globals()[f'Actuator_{dev_id}']
    if(dev_type == 2):
        del globals()[f'Driver_{dev_id}']
    elif(dev_type == 3):
        del globals()[f'IMU_{dev_id}']
    # Delete all objects with 0 reference counts
    gc.collect()
    
# TODO: Workout a flow of the program
# TODO: Handle C++ exceptions properly in Python: Includes Argument mismatch as well.
# TODO: Handle multiple device types properly
# TODO: Add code to run without IPython
if __name__ == '__main__':
    __print_banner()
    
    # Create required global variables
    __logger = Logger()
    __devManager = DeviceManager(__logger, __on_new_device_callback, __on_remove_device_callback)
    plotter = LivePlotter(__logger)
      
    # Create and Start shell
    __console = IPython.terminal.embed.InteractiveShellEmbed(banner1='')
    __console()
    
    # Stop all threads and exit
    __devManager.stopThreads()
    
    